import os
import subprocess

test_iteration = 0
tests_passed = 0

def executetest(input: str, expectedstdout: str | None, expectedexitcode: int, testname: str | None):
	global test_iteration
	global tests_passed
	test_iteration += 1

	file = open("testlogs/test-" + str(test_iteration) + ".log", "w") # hack!!
	process = subprocess.Popen(["./main"], shell=True, stdin=subprocess.PIPE, stdout=file, stderr=subprocess.PIPE)
	process.communicate(bytes(input, "utf-8"))
	process.wait()

	file.close()
	stdout = open("testlogs/test-" + str(test_iteration) + ".log", "r").read().strip()
	exitcode = process.returncode

	if (testname != None):
		print("Running [" + testname + "] test...")
	else:
		print("Running #" + str(test_iteration) + " test...")
	
	if (expectedstdout != None and (stdout != expectedstdout and exitcode != expectedexitcode)):
		print("\033[91mFAILURE: Unexpected output and exit code!")
		print("- Expected:\n" + expectedstdout + "\n--------------\n- Got:\n" + stdout)
		print("- Expected code: " + str(expectedexitcode) + ", Got code: " + str(exitcode) + "\033[00m")
	elif (expectedstdout != None and stdout != expectedstdout):
		print("\033[91mFAILURE: Unexpected output!\n- Expected:\n" + expectedstdout + "\n--------------\n- Got:\n" + stdout + "\033[00m")
	elif (exitcode != expectedexitcode):
		print("\033[91mFAILURE: Unexpected exit code!\n- (Expected code: " + str(expectedexitcode) + ", Got code: " + str(exitcode) + ")\033[00m")
	else:
		print("\033[92mPASSED!\033[00m")
		tests_passed += 1

testexpectedoutputs = [ # Store expected stdout outputs for tests that require it
	"""Enter what shape you want to calc: 
Enter what attribute you want to calc: 
Enter radius of the circle: 
78.539816""",
	"""Enter what shape you want to calc: 
Enter what attribute you want to calc: 
Enter radius of the circle: 
31.415927""",
	"""Enter what shape you want to calc: 
Enter what attribute you want to calc: 
Enter width of the square: 
25.000000""",
	"""Enter what shape you want to calc: 
Enter what attribute you want to calc: 
Enter width of the square: 
20.000000""",
	"""Enter what shape you want to calc: 
Enter what attribute you want to calc: 
Enter width of the cuboid: 
Enter length of the cuboid: 
Enter height of the cuboid: 
125.000000""",
	"""Enter what shape you want to calc: 
Enter what attribute you want to calc: 
Enter width of the cuboid: 
Enter length of the cuboid: 
Enter height of the cuboid: 
150.000000""",
]

def main():
	print("Compiling...")
	
	exitcode = os.system("gcc -o main main.c -lm")
	if exitcode != 0:
		print("Compilation failed!")
		return

	print("Starting tests...")
	# executetest function params:
	# <Stdin, <Expected stdout>, <Expected exitcode>, <Testname>
	# \r\n = Enter key

	# normal tests
	executetest("circle\r\narea\r\n5\r\n", testexpectedoutputs[0], 0, "Circle Area")
	executetest("circle\r\ncircumference\r\n5\r\n", testexpectedoutputs[1], 0, "Circle Circumference")
	
	executetest("square\r\narea\r\n5\r\n", testexpectedoutputs[2], 0, "Square Area")
	executetest("square\r\nperimeter\r\n5\r\n", testexpectedoutputs[3], 0, "Square Perimeter")

	executetest("cuboid\r\nvolume\r\n5\r\n5\r\n5\r\n", testexpectedoutputs[4], 0, "Cuboid Volume")
	executetest("cuboid\r\nsurface\r\n5\r\n5\r\n5\r\n", testexpectedoutputs[5], 0, "Cuboid Surface Area")

	# bad entries, program must always exit with code 1 (failure)
	executetest("cicle\r\nempty\r\n", None, 1, "Misspelling rejection of circle")
	executetest("squar\r\nempty\r\n", None, 1, "Misspelling rejection of square")
	executetest("coboid\r\nempty\r\n", None, 1, "Misspelling rejection of cuboid")

	print("\nTests passed: " + str(tests_passed) + "/" + str(test_iteration))


if __name__ == "__main__":
	main()
